IPA_Workflow <- function(spreadsheet) {
  ##
  on.exit({
    if ((geterrmessage()[1] != "") & (geterrmessage()[1] != "StopIteration")) { # StopIteration is generated by `foreach`
      IPA_logRecorder(geterrmessage())
      IPA_logRecorder("Stopped IDSL.IPA workflow!")
    }
    if (!is.null(warnings())) {
      IPA_logRecorder(warnings())
    }
  })
  ##
  initiation_time <- Sys.time()
  ##
  gc()
  closeAllConnections()
  ##
  PARAM <- IPA_xlsxAnalyzer(spreadsheet)
  if (!is.null(PARAM)) {
    ##
    timeZone <- tryCatch(Sys.timezone(), warning = function(w) {"UTC"}, error = function(e) {"UTC"})
    ##
    if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0005'), 2]) == "no") {
      ##
      ##########################################################################
      ## To create log record for IDSL.IPA
      input_path <- PARAM[which(PARAM[, 1] == 'PARAM0007'), 2]
      output_path <- PARAM[which(PARAM[, 1] == 'PARAM0010'), 2]
      .GlobalEnv$logIPA <- paste0(output_path, "/logIPA_performance.txt")
      IPA_logRecorder(paste0(rep("", 100), collapse = "="))
      IPA_logRecorder(paste0("mzML/mzXML/netCDF:  ", input_path))
      IPA_logRecorder(paste0("OUTPUT:  ", output_path))
      IPA_logRecorder(paste0(rep("", 100), collapse = "-"))
      IPA_logRecorder("Initiated IDSL.IPA workflow!")
      IPA_logRecorder(paste0(as.character(initiation_time), " ", timeZone))
      IPA_logRecorder("", printMessage = FALSE)
      IPA_logRecorder("", printMessage = FALSE)
      IPA_logRecorder(paste0(PARAM[, 1], "\t", PARAM[, 2]),  printMessage = FALSE)
      IPA_logRecorder(paste0(rep("", 100), collapse = "-"))
      ##
      ##########################################################################
      ##
      if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0001'), 2]) == "yes") {
        IPA_PeakAnalyzer(PARAM)
      }
      if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0002'), 2]) == "yes") {
        IPA_PeakAlignment(PARAM)
      } else {
        x0004 <- PARAM[which(PARAM[, 1] == 'PARAM0004'), 2]
        x0045 <- PARAM[which(PARAM[, 1] == 'PARAM0045'), 2]
        if (!is.na(x0004) & !is.na(x0045)) {
          if (tolower(x0004) == "yes" & tolower(x0045) == "yes") {
            IPA_PeakAlignment(PARAM)
          }
        }
      }
      if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0003'), 2]) == "yes") {
        IPA_GapFiller(PARAM)
      }
      if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0004'), 2]) == "yes") {
        if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0046'), 2]) == "yes") {
          IPA_CompoundsAnnotation(PARAM)
        }
        if (tolower(PARAM[which(PARAM[, 1] == 'PARAM0047'), 2]) == "yes") {
          IPA_PeaklistAnnotation(PARAM)
        }
      }
      ##
    } else {
      PARAM_targeted <- xlsxAnalyzer_EIC(spreadsheet)
      if (!is.null(PARAM_targeted)) {
        ##
        ##########################################################################
        ## To create log record for IDSL.IPA
        input_path <- PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM0007'), 2]
        output_path <- PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM0010'), 2]
        .GlobalEnv$logIPA <- paste0(output_path, "/logIPA_targeted.txt")
        IPA_logRecorder(paste0(rep("", 100), collapse = "="))
        IPA_logRecorder(paste0("mzML/mzXML/netCDF:  ", input_path))
        IPA_logRecorder(paste0("OUTPUT:  ", output_path))
        IPA_logRecorder(paste0(rep("", 100), collapse = "-"))
        IPA_logRecorder("Initiated the targeted analysis!")
        IPA_logRecorder(paste0(as.character(initiation_time), " ", timeZone))
        IPA_logRecorder("", printMessage = FALSE)
        IPA_logRecorder("", printMessage = FALSE)
        IPA_logRecorder(paste0(PARAM_targeted[, 1], "\t", PARAM_targeted[, 2]),  printMessage = FALSE)
        IPA_logRecorder(paste0(rep("", 100), collapse = "-"))
        ##
        ##########################################################################
        ##
        mzCandidate <- tryCatch(eval(parse(text = paste0("c(", PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM_MZ'), 2], ")"))), error = function(e){NULL})
        rtCandidate <- tryCatch(eval(parse(text = paste0("c(", PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM_RT'), 2], ")"))), error = function(e){NULL})
        #
        if (is.null(mzCandidate) | is.null(rtCandidate)) {
          stop(IPA_logRecorder("ERROR!!! Incorrect 'PARAM_MZ' or 'PARAM_RT'"))
        }
        ##
        ipa_eic_tar <- tolower(PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM_EIC'), 2])
        if (ipa_eic_tar == "y" | ipa_eic_tar == "yes") {
          exportEICcheck <- TRUE
        } else if (ipa_eic_tar == "n" | ipa_eic_tar == "no") {
          exportEICcheck <- FALSE
        }
        ##
        ipa_tab_tar <- tolower(PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM_CCT'), 2])
        if (ipa_tab_tar == "y" | ipa_tab_tar == "yes") {
          exportTableCheck <- TRUE
        } else if (ipa_tab_tar == "n" | ipa_tab_tar == "no") {
          exportTableCheck <- FALSE
        }
        ##
        IPA_TargetedTable <- IPA_TargetedAnalysis(spreadsheet, mzCandidate, rtCandidate, exportEIC = exportEICcheck, exportTable = exportTableCheck)
        ##
        if (exportTableCheck) {
          output_path <- PARAM_targeted[which(PARAM_targeted[, 1] == 'PARAM0010'), 2]
          save(IPA_TargetedTable, file = paste0(output_path, "/IPA_TargetedTable.Rdata"))
          write.csv(IPA_TargetedTable, file = paste0(output_path, "/IPA_TargetedTable.csv"))
          IPA_logRecorder(paste0("Targeted peaklists are stored in `.Rdata` and `.csv` formats in the `", output_path,"` folder!"))
        }
        ##
        IPA_logRecorder("Completed the targeted analysis!")
        IPA_logRecorder(paste0(rep("", 100), collapse = "-"))
      }
    }
    ##
    ##########################################################################
    ##
    completion_time <- Sys.time()
    IPA_logRecorder(paste0(rep("", 100), collapse = "-"))
    required_time <- completion_time - initiation_time
    print(required_time)
    IPA_logRecorder(paste0(as.character(completion_time), " ", timeZone), printMessage = FALSE)
    IPA_logRecorder("", printMessage = FALSE)
    IPA_logRecorder("", printMessage = FALSE)
    IPA_logRecorder("Completed IDSL.IPA computations successfully!")
    IPA_logRecorder(paste0(rep("", 100), collapse = "="), printMessage = FALSE)
    ##
    ##########################################################################
    ##
  }
  ##
  gc()
  closeAllConnections()
  ##
}
